var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { html, nothing } from "lit";
import { property } from "lit/decorators.js";
import { Debouncer, FtLitElementRedux, redux } from "@fluid-topics/ft-wc-utils";
import { FtReaderStateManager } from "./store/FtReaderStateManager";
import { styles } from "./ft-reader-context.styles";
import { ftAppInfoStore, userAssetsActions } from "@fluid-topics/ft-app-context";
import { ForbiddenErrorEvent, LoadErrorEvent, MapLoadedEvent, NavigationDoneEvent, NotFoundErrorEvent, TopicRelativeNotFoundEvent, UnauthorizedErrorEvent, VisibleTopicsChangeEvent } from "./models";
import { ReaderService } from "./services/ReaderService";
const fromNumberAttribute = (defaultValue) => (value) => +(value !== null && value !== void 0 ? value : "NaN") > 0 ? +value : defaultValue;
class FtReaderContext extends FtLitElementRedux {
    constructor() {
        super();
        this.visibleTopics = [];
        this.scrollToTargetDebouncer = new Debouncer(5);
        this.navigationDebouncer = new Debouncer(10);
        this.mapLoadedEventDebouncer = new Debouncer(50);
        this.addStore(ftAppInfoStore);
    }
    render() {
        return html `
            ${this.importStyleIfNeeded()}
            <slot @register-ft-reader-component=${this.registerComponent}></slot>
        `;
    }
    importStyleIfNeeded() {
        var _a, _b;
        const themeStylesheet = (_a = this.stateManager.service) === null || _a === void 0 ? void 0 : _a.themeStylesheet();
        if (!themeStylesheet || window.location.href.startsWith((_b = this.baseUrl) !== null && _b !== void 0 ? _b : "")) {
            return nothing;
        }
        return html `
            <style>
                @import "${themeStylesheet}"
            </style>
        `;
    }
    get stateManager() {
        if (this._stateManager == null) {
            this._stateManager = FtReaderStateManager.build(this.id || ("" + Math.round(Math.random() * 999000)).padStart(6, "0"), this.readerServiceProvider);
            this._stateManager.errorHandler = (e) => this.handleLoadError(e);
            this.addStore(this.stateManager.store, "reader");
        }
        return this._stateManager;
    }
    connectedCallback() {
        super.connectedCallback();
        this.stateManager.setMapId(this.mapId);
        this.stateManager.awaitService().then(service => {
            const fontsStylesheet = service.fontsStylesheet();
            if (fontsStylesheet && !window.location.href.startsWith(this.baseUrl) && !document.querySelector(`head link[href="${fontsStylesheet}"]`)) {
                const fonts = document.createElement("link");
                fonts.setAttribute("rel", "stylesheet");
                fonts.setAttribute("href", fontsStylesheet);
                document.head.append(fonts);
            }
            window.FluidTopicsReaderService = new ReaderService(this.stateManager);
        });
    }
    update(props) {
        super.update(props);
        if (props.has("readerServiceProvider") && this.readerServiceProvider) {
            this.stateManager.initService(this.readerServiceProvider);
        }
        if (props.has("mapId") && this.mapId) {
            this.stateManager.setMapId(this.mapId);
            this.lastNotifiedNotFoundRelative = undefined;
        }
        if (props.has("searchInDocumentQuery") && this.searchInDocumentQuery != null) {
            this.stateManager.setSearchInDocumentQuery(this.searchInDocumentQuery);
        }
        if (["tocId", "section", "page", "mapId", "topicPivot"].some(p => props.has(p)) && this.mapId) {
            this.navigateToTarget();
        }
        if (props.has("session")) {
            this.reload();
        }
    }
    contentAvailableCallback(props) {
        var _a, _b;
        super.contentAvailableCallback(props);
        if (props.has("map") || props.has("toc")) {
            this.sendLoadedEventIfLoaded();
        }
        if (props.has("scrollTarget") && ((_a = this.scrollTarget) === null || _a === void 0 ? void 0 : _a.tocId) == null) {
            const oldScrollTarget = props.get("scrollTarget");
            if ((oldScrollTarget === null || oldScrollTarget === void 0 ? void 0 : oldScrollTarget.tocId) != null) {
                this.scrollToTargetDebouncer.run(() => {
                    var _a;
                    return this.stateManager.service.getAccurateNavigationData(oldScrollTarget.tocId, (_a = this.currentPage) === null || _a === void 0 ? void 0 : _a.number, oldScrollTarget.section)
                        .then(data => this.dispatchEvent(new NavigationDoneEvent(data)));
                });
            }
        }
        if ((props.has("visibleTopics") || props.has("scrollTarget")) && ((_b = this.scrollTarget) === null || _b === void 0 ? void 0 : _b.tocId) == null) {
            Promise.all(this.visibleTopics.map(tocId => { var _a; return this.stateManager.service.getAccurateNavigationData(tocId, (_a = this.currentPage) === null || _a === void 0 ? void 0 : _a.number); }))
                .then(navigationData => {
                let visibleTopics = navigationData.filter(data => data != null && data.topicTitle !== "")
                    .map(data => {
                    var _a;
                    return ({
                        tocId: data.tocId,
                        title: data.topicTitle,
                        page: data.page,
                        prettyUrl: data.prettyUrl,
                        prettyUrlDuplicated: data.prettyUrlDuplicated,
                        breadcrumb: (_a = this.stateManager.service) === null || _a === void 0 ? void 0 : _a.buildBreadcrumb(data.tocId)
                    });
                });
                this.dispatchEvent(new VisibleTopicsChangeEvent(this.map, visibleTopics));
            });
        }
    }
    clear() {
        this.stateManager.clear();
    }
    async reload() {
        await this.stateManager.reload();
    }
    async navigateToTarget() {
        await this.navigationDebouncer.run(async () => {
            if (this.topicPivot != null) {
                const foundRelative = await this.stateManager.navigateToRelativeTopicIfFound(this.topicPivot);
                if (!foundRelative && this.lastNotifiedNotFoundRelative !== this.mapId + this.topicPivot) {
                    this.dispatchEvent(new TopicRelativeNotFoundEvent());
                    this.lastNotifiedNotFoundRelative = this.mapId + this.topicPivot;
                }
                else {
                    this.lastNotifiedNotFoundRelative = undefined;
                }
            }
            else if (this.tocId != null) {
                await this.stateManager.navigateToTopic(this.tocId, this.section, this.page);
            }
            else if (this.page != null) {
                await this.stateManager.navigateToPage(this.page);
            }
        });
    }
    registerComponent(e) {
        e.stopPropagation();
        const component = e.composedPath()[0];
        this.register(component);
    }
    register(component) {
        component.setReaderStateManager(this.stateManager);
    }
    handleLoadError(e) {
        const maybeHttpError = e;
        if (maybeHttpError.status === 401) {
            this.dispatchEvent(new UnauthorizedErrorEvent());
        }
        else if (maybeHttpError.status === 403) {
            this.dispatchEvent(new ForbiddenErrorEvent());
        }
        else if (maybeHttpError.status === 404) {
            this.dispatchEvent(new NotFoundErrorEvent());
        }
        else {
            this.dispatchEvent(new LoadErrorEvent(e));
        }
    }
    // Used in GWT
    reloadBookmarks() {
        userAssetsActions.reloadBookmarks();
    }
    // Used in GWT
    async reloadRating() {
        var _a, _b;
        await ((_a = this.stateManager.service) === null || _a === void 0 ? void 0 : _a.reloadRating());
        (_b = this.stateManager.store) === null || _b === void 0 ? void 0 : _b.dispatchEvent(new CustomEvent("rating-changed"));
    }
    // Used in GWT
    sendLoadedEventIfLoaded() {
        this.mapLoadedEventDebouncer.run(() => {
            if (this.map && this.toc) {
                this.dispatchEvent(new MapLoadedEvent(this.map, this.toc));
            }
        });
    }
}
// language=CSS
FtReaderContext.styles = styles;
__decorate([
    redux({ store: ftAppInfoStore.name })
], FtReaderContext.prototype, "baseUrl", void 0);
__decorate([
    redux({ store: ftAppInfoStore.name })
], FtReaderContext.prototype, "session", void 0);
__decorate([
    property()
], FtReaderContext.prototype, "mapId", void 0);
__decorate([
    property()
], FtReaderContext.prototype, "tocId", void 0);
__decorate([
    property()
], FtReaderContext.prototype, "section", void 0);
__decorate([
    property()
], FtReaderContext.prototype, "topicPivot", void 0);
__decorate([
    property()
], FtReaderContext.prototype, "searchInDocumentQuery", void 0);
__decorate([
    property({ type: Number, converter: { fromAttribute: fromNumberAttribute() } })
], FtReaderContext.prototype, "page", void 0);
__decorate([
    redux({ store: "reader" })
], FtReaderContext.prototype, "map", void 0);
__decorate([
    redux({ store: "reader" })
], FtReaderContext.prototype, "toc", void 0);
__decorate([
    redux({ store: "reader" })
], FtReaderContext.prototype, "visibleTopics", void 0);
__decorate([
    redux({ store: "reader" })
], FtReaderContext.prototype, "currentPage", void 0);
__decorate([
    redux({ store: "reader" })
], FtReaderContext.prototype, "scrollTarget", void 0);
__decorate([
    property({ attribute: false })
], FtReaderContext.prototype, "readerServiceProvider", void 0);
export { FtReaderContext };
