import { FtReaderConverter } from "./FtReaderConverter";
import { FtReaderService } from "./FtReaderService";
export class FtOfficialReaderService extends FtReaderService {
    static async build(mapId) {
        return new FtOfficialReaderService(new FtReaderConverter(), undefined, mapId);
    }
    clear() {
        super.clear();
        this.fingerprint = undefined;
    }
    async fetchMap(mapId) {
        const map = await this.api.getMap(mapId);
        this.fingerprint = map.fingerprint;
        return map;
    }
    fetchMapRating(mapId) {
        return this.api.getMapRating(mapId);
    }
    async fetchPages(mapId) {
        await this.getMap();
        return this.api.getPages(mapId, this.fingerprint);
    }
    fetchTopicHTMLContent(tocNode) {
        return this.api.getTopicHTMLContent(this.mapId, tocNode.contentId, "DESIGNED_READER", this.fingerprint);
    }
    fetchTopicMetadata(tocNode) {
        return this.api.getTopic(this.mapId, tocNode.contentId, this.fingerprint);
    }
    fetchAttachments() {
        return this.api.getMapAttachments(this.mapId).catch(() => []);
    }
    fetchRelativesForDocument(metadataPivotKey, metadataPivotValues) {
        const request = {
            query: "",
            paging: { page: 1, perPage: -1 },
            scope: "DOCUMENTS",
            metadataFilters: [
                {
                    key: metadataPivotKey,
                    valueFilter: {
                        negative: false,
                        values: metadataPivotValues,
                    }
                }
            ],
            sort: [],
            facets: []
        };
        return this.api.search(request);
    }
    fetchRelativesForTopicInMap(mapId, metadataPivotKey, metadataPivotValues) {
        const request = {
            query: "",
            paging: { page: 1, perPage: 1 },
            scope: "ALL_TOPICS",
            metadataFilters: [
                {
                    key: metadataPivotKey,
                    valueFilter: {
                        negative: false,
                        values: metadataPivotValues,
                    }
                },
                {
                    key: "ft:publicationId",
                    valueFilter: {
                        negative: false,
                        values: [mapId],
                    }
                }
            ],
            sort: [],
            facets: []
        };
        return this.api.search(request);
    }
    makeMapFeedbackRequest(message, from) {
        return this.api.sendMapFeedback(this.mapId, message, from);
    }
    makeRateMapRequest(type, value) {
        return this.api.rateMap(this.mapId, type, value);
    }
    makeRateTopicRequest(tocId, type, value) {
        return this.api.rateTopic(this.mapId, tocId, type, value);
    }
    makeTopicFeedbackRequest(tocId, message, from) {
        return this.api.sendTopicFeedback(this.mapId, tocId, message, from);
    }
    makeUnrateMapRequest() {
        return this.api.unrateMap(this.mapId);
    }
    makeUnrateTopicRequest(tocId) {
        return this.api.unrateTopic(this.mapId, tocId);
    }
    getReaderLinkPrefix() {
        return "r/";
    }
    isFeatureAvailable(feature) {
        return true;
    }
}
