import type { FluidTopicsApi, FtMap, FtMapAttachment, FtPaginatedToc, FtPaginationConfiguration, FtPublicationRatingSummary, FtRatingSummary, FtRatingTypeKeys, FtReaderConfiguration, FtTocNode, FtTopic } from "@fluid-topics/public-api";
import { FtRatingType, FtSearchResults } from "@fluid-topics/public-api";
import { FtPagesTocNode, FtReaderFeatureAccessData, FtReaderFeatures, FtReaderNavigationData, FtReaderPage, FtReaderTocNode, FtTopicRatingSummary } from "../model";
import { EnrichedToc, FtReaderConverter } from "./FtReaderConverter";
import { FtServiceWithCache } from "@fluid-topics/ft-app-context";
import { FtReaderBreadcrumb } from "../../models";
export declare abstract class FtReaderService extends FtServiceWithCache {
    protected converter: FtReaderConverter;
    protected abstract fetchMap(mapId: string): Promise<FtMap>;
    protected abstract fetchPages(mapId: string): Promise<FtPaginatedToc>;
    protected abstract fetchMapRating(mapId: string): Promise<FtRatingSummary>;
    protected abstract fetchTopicMetadata(tocNode: FtTocNode): Promise<FtTopic>;
    protected abstract fetchTopicHTMLContent(tocNode: FtTocNode): Promise<string>;
    protected abstract fetchAttachments(): Promise<Array<FtMapAttachment>>;
    protected abstract fetchRelativesForDocument(metadataPivotKey: string, metadataPivotValues: string[]): Promise<FtSearchResults>;
    protected abstract fetchRelativesForTopicInMap(mapId: string, metadataPivotKey: string, metadataPivotValues: string[]): Promise<FtSearchResults>;
    protected abstract makeRateMapRequest(type: FtRatingType | FtRatingTypeKeys, value: number): Promise<void>;
    protected abstract makeUnrateMapRequest(): Promise<void>;
    protected abstract makeMapFeedbackRequest(message: string, from?: string): Promise<void>;
    protected abstract makeRateTopicRequest(tocId: string, type: FtRatingType | FtRatingTypeKeys, value: number): Promise<void>;
    protected abstract makeUnrateTopicRequest(tocId: string): Promise<void>;
    protected abstract makeTopicFeedbackRequest(tocId: string, message: string, from?: string): Promise<void>;
    protected abstract getReaderLinkPrefix(): string;
    protected abstract isFeatureAvailable(feature: FtReaderFeatures): boolean;
    private _mapId;
    get mapId(): string;
    set mapId(value: string | undefined);
    constructor(converter: FtReaderConverter, overrideApi?: FluidTopicsApi, mapId?: string);
    private withMapId;
    private initCache;
    clear(): void;
    getMap(): Promise<FtMap>;
    getMapNow(): FtMap | undefined;
    getConfiguration(): Promise<FtReaderConfiguration>;
    private getEnrichedToc;
    get enrichedToc(): EnrichedToc | undefined;
    getPaginationConfiguration(): Promise<FtPaginationConfiguration>;
    getToc(): Promise<Array<FtReaderTocNode>>;
    getPagesToc(): Promise<Array<FtPagesTocNode>>;
    getTocNode(tocId: string): Promise<FtReaderTocNode>;
    getTocNodeNow(tocId: string): FtReaderTocNode | undefined;
    getTocNodeNowByUrl(prettyUrlOrMapIdTocId: string): FtReaderTocNode | undefined;
    getPage(number: number): Promise<FtReaderPage>;
    getPageByTocId(tocId: string): Promise<FtReaderPage>;
    getTopicInfo(tocNode: FtTocNode): Promise<FtTopic>;
    getTopicContent(tocNode: FtTocNode): Promise<string>;
    getAttachments(): Promise<Array<FtMapAttachment>>;
    getRelativesForDocument(metadataPivotKey: string, metadataPivotValues: string[]): Promise<FtSearchResults>;
    getRelativesForTopicInMap(mapId: string, metadataPivotKey: string, metadataPivotValues: string[]): Promise<FtSearchResults>;
    getAttachmentDownloadLink(id: string): string;
    getLink(tocId?: string, pageNumber?: number, section?: string): string;
    getNavigationData(tocId?: string, pageNumber?: number, section?: string): FtReaderNavigationData;
    getAccurateNavigationData(tocId?: string, pageNumber?: number, section?: string): Promise<FtReaderNavigationData>;
    private getRatingSummary;
    reloadRating(): Promise<FtRatingSummary>;
    getMapRating(): Promise<FtPublicationRatingSummary | undefined>;
    getTopicRating(tocId: string): Promise<FtTopicRatingSummary | undefined>;
    rateMap(type: FtRatingType | FtRatingTypeKeys, value: number): Promise<void>;
    unrateMap(): Promise<void>;
    sendMapFeedback(message: string, from?: string): Promise<void>;
    rateTopic(tocId: string, type: FtRatingType | FtRatingTypeKeys, value: number): Promise<void>;
    unrateTopic(tocId: string): Promise<void>;
    private clearRatingIfNecessary;
    sendTopicFeedback(tocId: string, message: string, from?: string): Promise<void>;
    fontsStylesheet(): string;
    themeStylesheet(): string;
    buildBreadcrumb(tocId: string): Array<FtReaderBreadcrumb>;
    isFeatureAccessible(feature: FtReaderFeatures, { session }: FtReaderFeatureAccessData): boolean;
    makeAbsolute(endpoint: string): string;
}
